import unittest
import requests

from bold_retriever import bold_retriever as br


class TestBoldRetriever(unittest.TestCase):

    def setUp(self):
        self.db = "COX1_L640bp"

    def test_parse_bold_xml_country_none(self):
        # normal request
        seq_object = "AGCCTGAGCTGGAATAGTTGGTACTTCGTTAAGTATTATAATTCGAGCTGAATT" \
                     "AGGACACCCCGGTGCTTTAATTGGTGATGACCAAATTTATAATGTAATTGTTAC" \
                     "TGCTCATGCTTTTATTATAATTttttttATAGTAATACCTATTATAATTGGA"
        r = requests.get(
            url="http://boldsystems.org/index.php/Ids_xml",
            params={'db': 'COX1_L640bp', 'sequence': seq_object}
        )
        request = r.text
        id = "random_id"
        all_ids = []
        taxon_list = []

        all_ids, taxon_list = br.parse_bold_xml(request, seq_object, id,
                                                all_ids,
                                                taxon_list,
                                                )
        for i in all_ids:
            if i['tax_id'] == 'Nemorimyza posticata':
                results = i['collection_country']
        expected = 'None'
        self.assertEqual(results, expected)


if __name__ == "__main__":
    runner = unittest.TextTestRunner(verbosity=2)
    unittest.main(testRunner=runner)



import os
import unittest


from bold_retriever import bold_retriever as br


class TestGetTaxIDFromWeb(unittest.TestCase):

    #def setUp(self):

    def test_when_scrapping_taxon_name_is_fail(self):
        obj = {
            'bold_id': 'GMGRE1022-13',
        }
        expected = {
            'bold_id': 'GMGRE1022-13',
        }
        result = br.get_tax_id_from_web(obj)
        self.assertEqual(expected, result)

    def test_when_scrapping_taxon_name_is_not_fail(self):
        obj = {
            'bold_id': 'NEUFI079-11',
        }
        expected = {
            'bold_id': 'NEUFI079-11',
            'tax_id': 'Hemerobius pini',
        }
        result = br.get_tax_id_from_web(obj)
        self.assertEqual(expected, result)

    def test_when_scrapping_taxon_name_is_fail2(self):
        obj = {
            'bold_id': 'NEUFI079-11aaaaaaaaaaaaaaaaaaaaaa',
            }
        expected = {
            'bold_id': 'NEUFI079-11aaaaaaaaaaaaaaaaaaaaaa',
            }
        result = br.get_tax_id_from_web(obj)
        self.assertEqual(expected, result)



import unittest

from bold_retriever import bold_retriever as br


class TestBoldRetriever(unittest.TestCase):

    def test_taxon_data(self):
        taxID = '297370'
        obj = {'division': 'animal', 'taxID': taxID}
        obj['class'] = 'Insecta'
        obj['order'] = 'Diptera'
        obj['family'] = 'Limoniidae'
        obj['classification'] = 'true'
        results = br.taxon_data(obj)
        self.assertEqual(results, obj)

    def test_taxon_data_returns_false(self):
        taxID = '29737001928929'
        obj = {'classification': 'false', 'taxID': taxID}
        results = br.taxon_data(obj)
        self.assertEqual(results['classification'], 'false')


if __name__ == "__main__":
    runner = unittest.TextTestRunner(verbosity=2)
    unittest.main(testRunner=runner)



import codecs
import os
import unittest

import requests

from bold_retriever import bold_retriever as br


class TestBoldRetriever(unittest.TestCase):

    def setUp(self):
        self.db = "COX1_L640bp"

    def test_parse_bold_xml1(self):
        # normal request
        r = requests.get(
            url="http://boldsystems.org/index.php/Ids_xml",
            params={
                'db': 'COX1_L640bp',
                'sequence': 'AAAGAATTTTAATTCGAGCTGAATTAAGTCAACCAGGAATATTTAT'
                            'TGGAAATGACCAAATTTATAACGTAATTGTTACAGCTCATGCTTTT'
                            'ATTATAATTttttttATAGTAATACCTATTATAATT',
            }
        )
        request = r.text
        seq_object = 'AAAGAATTTTAATTCGAGCTGAATTAAGTCAACCAGGAATATTTATTGGAAAT' \
                     'GACCAAATTTATAACGTAATTGTTACAGCTCATGCTTTTATTATAATTttttt' \
                     'tATAGTAATACCTATTATAATT'
        id = 'IonX17_rvr_ZA2013-0055_HochstetterForland_28_7_2013_10_21_San' \
             'derling_juvenile_98;size=2'
        all_ids = []
        taxon_list = []

        # get only taxon_list
        results = br.parse_bold_xml(request, seq_object, id, all_ids,
                                    taxon_list)[1]
        expected = ['Diptera', 'Culicidae', 'Ochlerotatus impiger']
        self.assertEqual(results, expected)

    def test_parse_bold_xml2(self):
        # malformed XML returned from BOLD
        request = []
        seq_object = 'AAAGAATTTTAATTCGAGCTGAATTAAGTCAACCAGGAATATTTATTGGAAAT' \
                     'GACCAAATTTATAACGTAATTGTTACAGCTCATGCTTTTATTATAATTttttt' \
                     'tATAGTAATACCTATTATAATT'
        id = 'IonX17_rvr_ZA2013-0055_HochstetterForland_28_7_2013_10_21_San' \
             'derling_juvenile_98;size=2'
        all_ids = []
        taxon_list = []

        # get only taxon_list
        results = br.parse_bold_xml(request, seq_object, id, all_ids,
                                    taxon_list)[1]
        expected = []
        self.assertEqual(results, expected)

    def test_parse_bold_xml3(self):
        # malformed XML returned from BOLD
        request = None
        seq_object = 'AAAGAATTTTAATTCGAGCTGAATTAAGTCAACCAGGAATATTTATTGGAAAT' \
                     'GACCAAATTTATAACGTAATTGTTACAGCTCATGCTTTTATTATAATTttttt' \
                     'tATAGTAATACCTATTATAATT'
        id = 'IonX17_rvr_ZA2013-0055_HochstetterForland_28_7_2013_10_21_San' \
             'derling_juvenile_98;size=2'
        all_ids = []
        taxon_list = []

        # get only taxon_list
        results = br.parse_bold_xml(request, seq_object, id, all_ids,
                                    taxon_list)[1]
        expected = []
        self.assertEqual(results, expected)

    def test_parse_bold_xml4(self):
        # malformed XML returned from BOLD
        request = "TEXT"
        seq_object = 'AAAGAATTTTAATTCGAGCTGAATTAAGTCAACCAGGAATATTTATTGGAAAT' \
                     'GACCAAATTTATAACGTAATTGTTACAGCTCATGCTTTTATTATAATTttttt' \
                     'tATAGTAATACCTATTATAATT'
        id = 'IonX17_rvr_ZA2013-0055_HochstetterForland_28_7_2013_10_21_San' \
             'derling_juvenile_98;size=2'
        all_ids = []
        taxon_list = []

        # get only taxon_list
        results = br.parse_bold_xml(request, seq_object, id, all_ids,
                                    taxon_list)[1]
        expected = []
        self.assertEqual(results, expected)

    def test_parse_bold_xml5(self):
        # malformed XML returned from BOLD
        request = """<?xml version="1.0" encoding="UTF-8"?>
                     <!-- Edited by XMLSpy -->
                     <note>
                        <to>Tove</to>
                        <from>Jani</from>
                        <heading>Reminder</heading>
                        <body>Don't forget me this weekend!</body>
                     </note>
                  """
        seq_object = 'AAAGAATTTTAATTCGAGCTGAATTAAGTCAACCAGGAATATTTATTGGAAAT' \
                     'GACCAAATTTATAACGTAATTGTTACAGCTCATGCTTTTATTATAATTttttt' \
                     'tATAGTAATACCTATTATAATT'
        id = 'IonX17_rvr_ZA2013-0055_HochstetterForland_28_7_2013_10_21_San' \
             'derling_juvenile_98;size=2'
        all_ids = []
        taxon_list = []

        # get only taxon_list
        results = br.parse_bold_xml(request, seq_object, id, all_ids,
                                    taxon_list)[1]
        expected = []
        self.assertEqual(results, expected)

    def test_create_output_file(self):
        result = br.create_output_file("my_fasta_file.fas")
        expected = "my_fasta_file.fas_output.csv"
        self.assertEqual(result, expected)

    def test_create_output_file2(self):
        expected = "my_fasta_file.fas_output.csv"
        if os.path.isfile(expected):
            os.remove(expected)
        br.create_output_file("my_fasta_file.fas")
        self.assertTrue(os.path.isfile(expected))

    def test_process_classification(self):
        obj = {
            'classification': 'true',
            'class': 'Insecta',
            'order': 'Lepidoptera',
            'family': 'Nymphalidae',
        }
        expected = "Insecta,Lepidoptera,Nymphalidae\n"
        result = br.process_classification(obj)
        self.assertEqual(expected, result)

    def test_process_classification_class_none(self):
        obj = {
            'classification': 'true',
        }
        expected = "None,None,None\n"
        result = br.process_classification(obj)
        self.assertEqual(expected, result)

    def test_process_classification_false(self):
        obj = {
            'classification': 'false',
        }
        expected = "None,None,None\n"
        result = br.process_classification(obj)
        self.assertEqual(expected, result)

    def test_generate_output_content_for_file(self):
        output_filename = 'ionx13.fas_output.csv'
        if os.path.isfile(output_filename):
            os.remove(output_filename)
        fasta_file = os.path.join(
            os.path.abspath(os.path.dirname(__file__)),
            'ionx13.fas',
        )
        br.generate_output_content_for_file(
            output_filename,
            fasta_file,
            'COX1_SPECIES',
        )
        result = codecs.open(output_filename, "r", "utf-8").readlines()[0]
        expected = "FIPSO166-14,ionx13,0.9796,Finland,animal,Psocoptera,Ins" \
                   "ecta,Psocoptera,None"
        self.assertEqual(expected, result.strip())

    def test_get_tax_id_from_web(self):
        obj = {'division': 'animal',
               'classification': 'true',
               'seq': 'AATTTGATCAGGTTTAGTAGGAACTAGATTAAGTTTATTAATTCGAGCCGAATTAGGTCAACCAGGTTCATTAATTGGAGATGACCAAATTTATAATGTAATCGTAACTGCTCATGCATTTATTATAATTTTCTTCATAGTTATACCTATTGTTATTGGA',
               'similarity': '1',
               'class': u'Insecta',
               'collection_country': 'Finland',
               'taxID': u'107',
               'bold_id': 'NEUFI079-11',
               'order': u'Neuroptera',
               'id': 'OTU_99',
               'tax_id': 'Neuroptera'}
        results = br.get_tax_id_from_web(obj)
        self.assertEqual('Hemerobius pini', results['tax_id'])


if __name__ == "__main__":
    runner = unittest.TextTestRunner(verbosity=2)
    unittest.main(testRunner=runner)



import argparse
from argparse import RawTextHelpFormatter
import codecs
import json
import logging
import re
import xml.etree.ElementTree as ET

from Bio import SeqIO
from bs4 import BeautifulSoup
import requests


def parse_bold_xml(request, seq_object, id, all_ids, taxon_list):
    try:
        root = ET.fromstring(request)
        for match in root.findall('match'):
            out = dict()
            out['seq'] = str(seq_object)
            out['id'] = str(id)
            similarity = match.find('similarity').text
            out['similarity'] = similarity
            tax_id = match.find('taxonomicidentification').text
            out['tax_id'] = tax_id

            if match.find('specimen/collectionlocation/country').text:
                ctry = match.find('specimen/collectionlocation/country').text
                out['collection_country'] = ctry
            else:

                out['collection_country'] = "None"

            myid = match.find('ID').text
            out['bold_id'] = myid
            if not out['tax_id'] in taxon_list:
                taxon_list.append(out['tax_id'])
                all_ids.append(out)
        return all_ids, taxon_list
    except ET.ParseError as e:
        print "\n>> Error got malformed XML from BOLD: " + str(e)
        return all_ids, taxon_list
    except TypeError as e:
        print "\n>> Error got malformed XML from BOLD: " + str(e)
        return all_ids, taxon_list


def request_id(seq_object, id, db, debug=False):
    """
    Sends a sequence to BOLD REST API for identification using a database
    specified by the user.

    :param seq_object: sequence as string
    :param id: sequence id as string
    :param db: BOLD database specified by user as string
    :return: two lists of diciontaries with some identification info
    """
    all_ids = []
    taxon_list = []
    url = "http://boldsystems.org/index.php/Ids_xml"
    payload = {'db': db, 'sequence': str(seq_object)}

    r = get(url, payload)
    r_text = r.text
    if debug is True:
        r_text = []

    if isinstance(r_text, basestring):
        all_ids, taxon_list = parse_bold_xml(r_text, seq_object, id, all_ids,
                                             taxon_list)
        if all_ids is not None and len(all_ids) > 0:
            # for i in all_ids:
            # print i['tax_id'], i['similarity']
            return all_ids
        else:
            return None
    else:
        return None


def taxon_search(obj):
    # obj['tax_id'] = "Morpho helenor"
    tax_id = obj['tax_id'].split(" ")
    if len(tax_id) > 1:
        tax_id = tax_id[0]
    url = "http://www.boldsystems.org/index.php/API_Tax/TaxonSearch/"
    # print "I am sending this %s" % tax_id
    payload = {
        'taxName': tax_id,
        'fuzzy': 'false',
    }
    r = get(url, payload)
    found_division = False
    if r.text != "":
        response = json.loads(r.text)
        if hasattr(response, 'items'):
            for k, v in response.items():
                try:
                    if v['tax_division'] == 'Animals':
                        # this is the taxID
                        found_division = True
                        return {'division': 'animal', 'taxID': k}
                except:
                    logging.warning("Error: %s" % str(r.text))

            if not found_division:
                for k, v in json.loads(r.text).items():
                    try:
                        if v['tax_division'] != 'Animals':
                            # this is the taxID
                            found_division = True
                            return {'division': 'not animal', 'taxID': k}
                    except:
                        logging.warning("Got funny reply from BOLD: " % str(r.text))
        else:
            return None
    return None


def taxon_data(obj):
    this_tax_id = obj['taxID']
    url = "http://www.boldsystems.org/index.php/API_Tax/TaxonData/"
    payload = {'taxId': this_tax_id, 'dataTypes': 'basic',
               'includeTree': 'true'
               }
    req = get(url, payload)

    # this is a "list" then
    if req.text == '[]':
        obj = {'classification': 'false'}
        return obj
    # this is a string not a list
    elif isinstance(req.text, basestring):
        items = json.loads(req.text).items()
        for key, val in items:
            try:
                if val['tax_rank'] == 'class':
                    obj['class'] = val['taxon']
                if val['tax_rank'] == 'order':
                    obj['order'] = val['taxon']
                if val['tax_rank'] == 'family':
                    obj['family'] = val['taxon']
            except TypeError:
                logging.warning("Exception: No values for some of the keys.")
            obj['classification'] = "true"
        if 'family' in obj:
            return obj
        else:
            # The family name for this specimen is returned as `tax_id`
            # because this samples is not `public` by BOLD when using the API.
            # Try to get the tax_id from the webpage BIN.
            return get_tax_id_from_web(obj)


def get_tax_id_from_web(obj):
    """Try to get the tax_id from the webpage BIN."""
    logging.info('Trying to get the tax_id from the webpage Public_BIN.')

    # get cart token
    url = 'http://www.boldsystems.org/index.php/Public_BINSearch'
    payload = {
        'taxon': '',
        'searchMenu': 'bins',
        'query': obj['bold_id'],
    }
    r = get(url, payload)
    cart_token = re.search('= \'(general_.+)\';//', r.text).groups()[0]

    # get actual data using the cart token
    url = 'http://www.boldsystems.org/index.php/Public_Ajax_BinList'
    payload = {
        'offset': 0,
        'limit': 100,
        'query': obj['bold_id'],
        'cartToken': cart_token,
        'inc[]': 'ids::processid::' + obj['bold_id'],
        'contextOp': 'AND',
        '_': 1412165276887,
    }
    r = get(url, payload)
    soup = BeautifulSoup(r.text)
    for i in soup.find_all('span'):
        if 'Species' in i.get_text():
            taxon = i.next_sibling.string.strip()
            res = re.search('(\w+\s?\w*)\s\(', taxon)
            if res:
                obj['tax_id'] = res.groups()[0]
            return obj
    logging.info('The BOLD webpage does not contain Genus and Species names '
                 'for BOLD record %s.' % obj['bold_id'] )
    return obj


def create_parser():
    description = "send seqs to BOLD Systems API and retrieve results"
    parser = argparse.ArgumentParser(description=description,
                                     formatter_class=RawTextHelpFormatter,
                                     )
    parser.add_argument('-f', '--file', action='store', help='Fasta filename',
                        required=True, dest='fasta_file',
                        )
    parser.add_argument('-db', '--database', action='store',
                        help='Choose a BOLD database. Enter one option.',
                        choices=[
                            'COX1_SPECIES',
                            'COX1',
                            'COX1_SPECIES_PUBLIC',
                            'COX1_L640bp',
                        ],
                        required=True,
                        dest='db',
                        )
    return parser


def create_output_file(f):
    """Containing only column headers of the CSV file."""
    output = "bold_id,seq_id,similarity,collection_country,division,taxon,"
    output += "class,order,family\n"

    output_filename = f.strip() + "_output.csv"
    myfile = codecs.open(output_filename, "w", "utf-8")
    myfile.write(output)
    myfile.close()
    return output_filename


def process_classification(obj):
    out = ""
    if obj['classification'] == "true":
        if 'class' in obj:
            out += obj['class'] + ","
        else:
            out += "None,"

        if 'order' in obj:
            out += obj['order'] + ","
        else:
            out += "None,"

        if 'family' in obj:
            out += obj['family']
        else:
            out += "None"
        out += "\n"
    else:
        out += "None,None,None\n"
    return out


def generate_output_content_for_file(output_filename, fasta_file, db):
    for seq_record in SeqIO.parse(fasta_file, "fasta"):
        print("Processing sequence for %s" % str(seq_record.id))
        out = ""
        all_ids = request_id(seq_record.seq, seq_record.id, db)
        if all_ids is not None:
            for obj in all_ids:
                if 'tax_id' in obj:
                    r = taxon_search(obj)

                    if r is None:
                        continue
                    obj['taxID'] = r['taxID']
                    obj['division'] = r['division']
                    # print "== obj", obj
                    obj = taxon_data(obj)
                    out += obj['bold_id'] + ","
                    out += obj['id'] + "," + obj['similarity'] + ","
                    out += obj['collection_country'] + ","
                    out += obj['division'] + ","
                    out += obj['tax_id'] + ","
                    out += process_classification(obj)
            with codecs.open(output_filename, "a", "utf-8") as handle:
                handle.write(out)
        else:
            out = "nohit," + str(seq_record.id) + ","
            out += "nohit,nohit,nohit,nohit,nohit,nohit,nohit\n"
            with codecs.open(output_filename, "a", "utf-8") as handle:
                handle.write(out)
    print("Processed all sequences.")
    return "Processed all sequences."


def get(url, payload):
    """Wrapper function for requests.get so we can use fake requests when
    writing unittests.
    :param url:
    :param params: payload
    :return: response object from requests.get
    """
    r = requests.get(url, params=payload)
    return r


def get_args(args):
    db = args.db
    f = args.fasta_file
    return f, db


def get_started(args):
    fasta_file, db = get_args(args)
    output_filename = create_output_file(fasta_file)
    generate_output_content_for_file(output_filename, fasta_file, db)


def main():
    parser = create_parser()
    args = parser.parse_args()
    get_started(args)


if __name__ == "__main__":
    main()
